/*
=========================================================
 $Comments:     This routines is converted from ADFOpus
=========================================================
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "support.h"

#ifdef DEBUG
#include "debug.h"
#endif

#ifdef DIRSEP
#undef DIRSEP
#endif

#ifdef WIN32
#define DIRSEP '\\'
#else
#define DIRSEP '/'
#endif /* WIN32 */

char tempDir[] = "/tmp";

enum adfDiskFormat	VolGetFormat(char *fileName, int *Error)
// Checks input file for ADZ or DMS types.
// Output: returns an enumerated disk type ADF, ADZ or DMS where ADF covers all
//         non-compressed types i.e. disk dumps, hardfiles etc.
{
	char fileSuf[4];
	char *tempName = strdup(fileName);
	char tempFile[FILENAME_MAX * 3];
	char *pt = strrchr(tempName, '.'), *inName;
	if(pt) {
		strcpy(fileSuf, pt+1);
	}
	// Check for hardfile or dump. Return name unchanged if found.
	if(pt == NULL || strcmp(fileSuf, "dmp") == 0 ||
	   strcmp(fileSuf, "DMP") == 0 ||
	   strcmp(fileSuf, "hdf") == 0 ||
	   strcmp(fileSuf, "HDF") == 0 ||
	   strcmp(fileSuf, "adf") == 0 ||
	   strcmp(fileSuf, "ADF") == 0) {
	   	free(tempName);
		return ADF;
	}
	*pt--='\0';
	while(*pt && pt[-1]!=DIRSEP) pt--;
	sprintf(tempFile, "%s%c%s", tempDir, (int)DIRSEP, pt);

	if(strcmp(fileSuf, "dms") == 0 || strcmp(fileSuf, "DMS") == 0 ){	// Open DMS.
		inName = strdup(fileName);
		strcpy(fileName, tempFile);
		strcat(fileName, ".adf");
#ifdef DEBUG
	log("original:= %s, base:= %s, temp:= %s", fileName, pt, tempFile);
#endif
		if(dmsUnpack(inName, fileName) != NO_PROBLEM)
			*Error = TRUE;

		free(inName);
		free(tempName);
		return DMS;
	} else if(strcmp(fileSuf, "adz") == 0 || strcmp(fileSuf, "ADZ") == 0 ){	// Open ADZ.
		inName = strdup(fileName);
		strcpy(fileName, tempFile);
		strcat(fileName, ".adf");
#ifdef DEBUG
	log("original:= %s, base:= %s, temp:= %s", fileName, pt, tempFile);
#endif
		if(GZDecompress(inName, fileName) == 0)
			*Error = TRUE;

		free(inName);
		free(tempName);
		return ADZ;
	}

	free(tempName);
	return ADF;
}

#define BUFLEN      16384
int GZDecompress(char *infile, char *outfile)
// Decompress an adf compressed with gzip.
// Input: input file name, output file name
{
    gzFile	in;
    FILE	*out;
    char	buf[BUFLEN];
    int		len;

    out = fopen(outfile, "wb");
    if(out == NULL)
		return FALSE;

    in = gzopen(infile, "rb");
    if(in == NULL)
		return FALSE;

	for(;;){
        len = gzread(in, buf, sizeof(buf));
        if(len < 0)
			return FALSE;
        if(len == 0)
			break;
        if((int)fwrite(buf, 1, (unsigned)len, out) != len)
			return FALSE;
    }
    if(fclose(out))
		return FALSE;

    if(gzclose(in) != Z_OK)
		return FALSE;

	return TRUE;
}
